mod parser;
mod emitter;
mod internal;

use clap::Parser;

use std::io;
use std::io::prelude::*;
use std::fs::File;
use std::path::PathBuf;

#[derive(Parser, Debug)]
#[clap(author, version, about = "Compiler for Deelang")]
struct Cli {
    #[clap(help="Specify a file to compile")]
    file: PathBuf,
    #[clap(short, long, help="Emit a parse tree")]
    parse: bool,
    #[clap(short, long, help="Cross compile to ECMAScript")]
    ecmascript: bool,
    #[clap(long, help="Run the preprocessor")]
    preprocess: bool,
}

fn main() {
    let cli = Cli::parse();
    let mut file = File::open(&cli.file).expect("Couldn't read file");
    let mut prgm = String::new();
    file.read_to_string(&mut prgm).unwrap();
    if cli.preprocess {
	println!("{}", parser::preprocess(&prgm));
	return;
    }
    let tree = parser::parse(&prgm);
    if cli.parse {
        println!("{:#?}", tree);
    } else if cli.ecmascript {
	let mut out = io::stdout();
	emitter::emit_injector(&mut out).ok();
	let mut toplevel = emitter::LexicalContext::toplevel();
	emitter::emit_all(&mut out, &tree, &mut toplevel).ok();
    }
}
